/*
	module vvfstools.c
*/
/***********************************************************************
	vvfsfind.c - Find (child) node in tree. Search nodes at ALL levels.
		Recursive routine.

	terminate if			return
	--------------			------
	'root' is null			0
	No match found in subtree	0 
	match				Address of matching node.
************************************************************************/
#include	"vvfshdr.c"

TREENODE *vvfsfind ( root, frame )
TREENODE *root ;
char *frame ;
{
	TREENODE *p = 0 ;

	if (root == 0)		/* NULL subtree pointer passed */
	    ;
	else if ( strcmp ( root->child, frame ) == MATCH )
	    p = root ;
	else {
	    p = vvfsfind ( root->tn_child, frame ) ;	/* go deeper */

	    if ( p == 0 )	/* no match in above child subtree
				   so move horozontally */
		p = vvfsfind ( root->tn_sibling, frame ) ;
	}

	return ( p ) ;
}

/******************************************************************
	Quickly Locate a node in the tree. Recursive routine.

	terminate if			return
	--------------			------
	'root' is null			0
	No match found in subtree	0 
	match				p
*******************************************************************/
/*	#include	"vvfshdr.c" */

TREENODE *vvfsqfind ( root, frame, level )
TREENODE *root ;
char *frame ;
int level ;
{
	TREENODE *p = 0 ;

	if (root == 0)		/* NULL subtree pointer passed */
	    ;
	else if ( root->childlevel < level )
	{
	    p = vvfsqfind ( root->tn_child, frame, level ) ;	/* go deeper */
	    if ( p == 0 )	/* no match in above child subtree
				   so move horozontally */
		p = vvfsqfind ( root->tn_sibling, frame, level ) ;
	}
	else if ( root->childlevel == level )
	{
	    if ( strcmp ( root->child, frame ) == MATCH )
		/* we've got a hit! */
		p = root ;
	    else
		/* move horozontally */
		p = vvfsqfind ( root->tn_sibling, frame, level ) ;
	}
	else			/* too deep */
	    ;

	return ( p ) ;
}

/**********************************************************************
 vvfsstrim - Trim trailing white space from a string and NULL terminate.
	Resulting string is guaranteed to be NULL terminated, even if
	last byte in string must be overwritten with NULL. String is
	NOT left justified or compressed.

	Example call:
		char strng[10] ;
		strcpy ( strng, "String with trailing blanks.    ");
		vvfsstrim ( strng, sizeof ( strng ) ) ;

	Return codes: None.

	Author:	Pete Eakle, 4/25/86

**********************************************************************/
#include ctype

vvfsstrim ( str, len )
char *str ;
int len ;
{
	register int offset = len - 1 ; /* turn length into offset */

	while ( offset >= 0 )
	{
	    if ( !isspace ( str[offset] ) && str[offset] != '\0' )
	    {
		/* ASSERTION: we've hit a meaningful character

		   Back up 1 char & NULL terminate; Unless backing
		   up 1 char would put us before beginning of string.
		*/

		if ( offset == len - 1 )
		    str [ offset ] = '\0' ;	/* we're already at end
				of string. Put NULL right where we are.*/
		else
		{
		    offset++ ;
		    str [ offset ] = '\0' ;
		}
		break ;
	    }
	    else if ( offset == 0 )
	    {
		/* ASSERTION: entire string is blank and we're now
		   pointing at start of string. NULL terminate byte 0*/
		str [ offset ] = '\0' ;
	    }
	    offset-- ; 	/* walk backwards. */

	}
	return ;
}

/***************************************************************
	vvfsptree - print the menutree. recursive routine.
***************************************************************/
/*	#include	"vvfshdr.c" */

vvfsptree ( root )
TREENODE *root ;
{
	char *desctype() ;
	int i ;

	if ( !root )	/* NULL pointer passed in */
	    ;
	else
	{
	    for ( i = 0; i < (root->childlevel - 1) ; i++ )
		printf ("\t") ;

	    printf ("%s (%s), level=%d, '%s'\n",
		root->child, 
		desctype ( (int)root->type ),
		root->childlevel,
		root->itemdesc ) ;
	    vvfsptree ( root->tn_child ) ;
	    vvfsptree ( root->tn_sibling ) ;
	}
}
static char *desctype ( type )
int type ;
{
	switch ( type ) {
		case 1:
			return ( "menu" ) ;
		case 2:
			return ( "frame" ) ;
		case 3:
			return ( "procedure" ) ;
		default:
			return ( "unknown type") ;
		}
}
